@@ -28,10 +28,21 @@ try:
 except ImportError:
     pass
 
-__version__ = '0.10.0'
+__version__ = '0.11.1'
 __version_info__ = __version__.split('.')
 
-CTPP2_VERSION = (2, 6, 16)
+CTPP2_VERSION = (2, 7, 0)
+
+
+STRING_RE = r"""(?: '(?: [^'\\] | \\. )*' | "(?: [^"\\] | \\.)*")"""
+
+REGEXP = re.compile(r"""(?:
+(?: ((?: GETTEXT) | _) (\s*) [(] (\s*) ({0}) (\s*) , (\s*) ({0}) (\s*) , ) |
+(?: ((?: GETTEXT) | _) (\s*) [(] (\s*) ({0}) (\s*) , (\s*)  {0}  (\s*) [)] ) |
+(?: ((?: GETTEXT) | _) (\s*) [(] (\s*) ({0})                     (\s*) [)] ) |
+(\s*)
+)""".format(STRING_RE), re.IGNORECASE | re.VERBOSE)
+
 
 def extract_i18n_messages(fileobj, keywords, comment_tags, options):
     """Extract messages from XXX files.
@@ -46,42 +57,25 @@ def extract_i18n_messages(fileobj, keywords, comment_tags, options):
              tuples
     :rtype: ``iterator``
     """
-    string = r'(([^"]|(\"))+?)'
-    pattern = r"""
-        (_|GETTEXT)\s*
-        [(]\s*
-        " %s ((")|$)""" % string
-    regexp = re.compile(pattern, re.IGNORECASE|re.VERBOSE)
-    string_tail_pat = re.compile('^%s"' % string)
-    string_part_pat = re.compile('^%s$' % string)
 
-    lineno = 0
-    buffered = None
-    messages = []
-    for line in fileobj:
-        lineno += 1
-        if buffered:
-            matched = string_tail_pat.search(line)
-            if matched:
-                messages.append(buffered + '\n' + matched.group(1))
-                for message in messages:
-                    yield saved_lineno, None, message, []
-                messages = []
-                buffered = None
-            else:
-                matched = string_part_pat.search(line)
-                if matched:
-                    buffered += '\n' + matched.group(1)
+    def count_newlines(elems):
+        return sum([ elem.count('\n') for elem in elems if elem ])
+
+    data = fileobj.read()
+    lineno = 1
+    for matched in REGEXP.finditer(data):
+        lineno += count_newlines(matched.groups())
+        if not matched:
+            continue
+
+        messages = (m.strip(' \t\v\f\r') for m in matched.groups() if m)
+        messages = [m for m in messages if m]
+        if messages and (messages[0].upper() == 'GETTEXT' or messages[0] == '_'):
+            saved_lineno = lineno - count_newlines(messages)
+            messages = [m[1:-1] for m in messages[1:]]
+            func_name = 'gettext'
+            if len(messages) > 1:
+                func_name = 'ngettext'
+            yield saved_lineno, func_name, messages, []
 
-        if not buffered:
-            for matched in regexp.finditer(line):
-                if not matched.group(5):
-                    buffered = matched.group(2)
-                    saved_lineno = lineno
-                    break
-                messages.append(matched.group(2))
-            else:
-                for message in messages:
-                    yield lineno, None, message, []
-                messages = []
 
